@@ -28,7 +28,7 @@ from oslotest import base as test_base
 from oslotest import mockpatch
 import six
 
-from oslo.concurrency import processutils  # noqa
+from oslo.concurrency import processutils
 
 PROCESS_EXECUTION_ERROR_LOGGING_TEST = """#!/bin/bash
 exit 41"""
@@ -296,7 +296,7 @@ grep foo
 
         out, err = processutils.execute('/usr/bin/env', env_variables=env_vars)
 
-        self.assertIn(b'SUPER_UNIQUE_VAR=The answer is 42', out)
+        self.assertIn('SUPER_UNIQUE_VAR=The answer is 42', out)
 
     def test_exception_and_masking(self):
         tmpfilename = self.create_tempfiles(
@@ -427,7 +427,7 @@ class FakeSshChannel(object):
         return self.rc
 
 
-class FakeSshStream(six.StringIO):
+class FakeSshStream(six.BytesIO):
     def setup_channel(self, rc):
         self.channel = FakeSshChannel(rc)
 
@@ -437,11 +437,11 @@ class FakeSshConnection(object):
         self.rc = rc
 
     def exec_command(self, cmd):
-        stdout = FakeSshStream('stdout')
+        stdout = FakeSshStream(b'stdout')
         stdout.setup_channel(self.rc)
-        return (six.StringIO(),
+        return (six.BytesIO(),
                 stdout,
-                six.StringIO('stderr'))
+                six.BytesIO(b'stderr'))
 
 
 class SshExecuteTestCase(test_base.BaseTestCase):
@@ -466,13 +466,13 @@ class SshExecuteTestCase(test_base.BaseTestCase):
 
     def _test_compromising_ssh(self, rc, check):
         fixture = self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
-        fake_stdin = six.StringIO()
+        fake_stdin = six.BytesIO()
 
         fake_stdout = mock.Mock()
         fake_stdout.channel.recv_exit_status.return_value = rc
-        fake_stdout.read.return_value = 'password="secret"'
+        fake_stdout.read.return_value = b'password="secret"'
 
-        fake_stderr = six.StringIO('password="foobar"')
+        fake_stderr = six.BytesIO(b'password="foobar"')
 
         command = 'ls --password="bar"'
 
